import capstone
from elf_to_shellcode.lib import five

ARCHES = {
    "mips": capstone.CS_ARCH_MIPS,
    "x32": capstone.CS_ARCH_X86,
    "x64": capstone.CS_ARCH_X86,
    "arm64": capstone.CS_ARCH_ARM64,
    "arm32": capstone.CS_ARCH_ARM
}

ENDIAN = {
    "<": capstone.CS_MODE_LITTLE_ENDIAN,
    ">": capstone.CS_MODE_BIG_ENDIAN
}


class Disassembler(object):
    def __init__(self, shellcode):
        mode = capstone.CS_MODE_32
        if shellcode.arch in ['x64']:
            mode = capstone.CS_MODE_64
        if shellcode.arch in ["arm32", 'arm64']:
            mode = capstone.CS_MODE_ARM
        self.cs = capstone.Cs(
            ARCHES[shellcode.arch],
            ENDIAN[shellcode.endian] | mode
        )
        self.shellcode = shellcode
        offset = self.shellcode.instruction_offset_after_objdump
        self.opcodes = self.shellcode.do_objdump(self.shellcode.shellcode_data)[offset:]
        self.instructions = [instruction for instruction in self.cs.disasm(
            five.to_disasm(self.opcodes),
            offset,
        )]

    def get_instruction_addresses(self, instruction_filter):
        addresses = []
        for instruction in self.instructions:
            if instruction_filter(instruction):
                to_off = instruction.size
                to_off -= self.shellcode.ptr_size
                addresses.append(instruction.address + to_off)
        return addresses



from elf_to_shellcode.mips.mips import mips_make_shellcode
from elf_to_shellcode.intel.x32 import intel_x32_make_shellcode
from elf_to_shellcode.intel.x64 import intel_x64_make_shellcode
from elf_to_shellcode.arm.x32 import arm_x32_make_shellcode
from elf_to_shellcode.arm.x64 import arm_x64_make_shellcode
from elf_to_shellcode.lib.consts import StartFiles


class Arches(object):
    MIPS_32 = "mips"
    INTEL_X32 = "intel_x32"
    INTEL_X64 = "intel_x64"
    ARM_32 = "arm32"
    ARM_64 = "aarch64"
    __all__ = [
        MIPS_32,
        INTEL_X32,
        INTEL_X64,
        ARM_32,
        ARM_64
    ]


ENDIANS = ["big", "little"]

shellcode_handlers = {
    Arches.MIPS_32: mips_make_shellcode,
    Arches.INTEL_X32: intel_x32_make_shellcode,
    Arches.INTEL_X64: intel_x64_make_shellcode,
    Arches.ARM_32: arm_x32_make_shellcode,
    Arches.ARM_64: arm_x64_make_shellcode,
}


def make_shellcode(binary_path, arch, endian, start_file_method):
    assert endian in ENDIANS, 'Chose endain from: {}'.format(ENDIANS)
    assert arch in shellcode_handlers, 'Chose arch from: {}'.format(
        shellcode_handlers.keys()
    )
    assert start_file_method in StartFiles.__all__
    return shellcode_handlers[arch](binary_path, endian=endian,
                                    start_file_method=start_file_method)



class RelocationAttributes(object):
    call_to_resolve = 1
    relative_to_loader_base = 2
    relative = 3


class RELOC_TYPES(object):
    JMP_SLOT = "JMP_SLT"
    RELATIVE = "RELATIVE"
    GLOBAL_SYM = "GLOBAL_SYM"
    GLOBAL_DAT = "GLOBAL_DAT"
    DO_NOT_HANDLE = "DO_NOT_HANDLE"


class StartFiles(object):
    glibc = "glibc"
    no_start_files = "no"

    __all__ = [
        glibc,
        no_start_files
    ]


class LoaderSupports(object):
    choices = {
        "dynamic": ('dynamic', 0)
    }

    @staticmethod
    def resolve_choice(key):
        return LoaderSupports.choices[key]


class OUTPUT_FORMAT_MAP(object):
    eshelf = 'eshelf'
    shelf = 'shelf'


OUTPUT_FORMATS = [OUTPUT_FORMAT_MAP.eshelf, OUTPUT_FORMAT_MAP.shelf]



import sys

version = int(sys.version[0])
is_python3 = version == 3


def to_disasm(obj):
    if is_python3:
        if type(obj) is not bytes:
            obj = obj.encode("utf-8")
        return obj
    return obj


def to_file(obj):
    if is_python3:
        return bytes(obj)
    return obj


def py_obj():
    if is_python3:
        return b''
    else:
        return ''


def array_join(array):
    joined = py_obj()
    for obj in array:
        joined += obj
    return joined


def ljust(source, size, by):
    length = (size - len(source))
    if length < 0:
        length = 0
    source += by * length
    assert len(source) == size
    return source


from elf_to_shellcode.lib.shellcode import Shellcode, create_make_shellcode


class ArmX32Shellcode(Shellcode):
    def __init__(self, elffile, shellcode_data, endian, **kwargs):
        super(ArmX32Shellcode, self).__init__(
            elffile=elffile,
            shellcode_data=shellcode_data,
            endian=endian,
            arch="arm32",
            mini_loader_little_endian="mini_loader_arm_x32{}.shellcode",
            mini_loader_big_endian="mini_loader_arm_x32be{}.shellcode",
            shellcode_table_magic=0xaabbccdd,
            ptr_fmt="I",
            sections_to_relocate={
                '.data.rel.ro': {'align_by': 'sh_addralign'},
                '.got.plt': {'align_by': 'sh_entsize', 'relocate_all': True},
            },
            support_dynamic=False,
            **kwargs
        )


arm_x32_make_shellcode = create_make_shellcode(ArmX32Shellcode)



class AddressUtils(object):
    def __init__(self, unpack_size):
        self.unpack_size = unpack_size

    def section_get_ptr_at_address(self, section, address, alignment):
        start = section.header.sh_addr
        end = start + section.header.sh_size
        assert address < end, 'Error, address: {} out of range'.format(address)
        index_start = address - start
        index_end = index_start + alignment

        return self.unpack_size(
            size=alignment,
            data=section.data()[index_start:index_end]
        )



import sys
from elf_to_shellcode.relocate import make_shellcode, Arches, ENDIANS, StartFiles
import logging
from argparse import ArgumentParser
from elf_to_shellcode.lib.consts import LoaderSupports, OUTPUT_FORMATS
from elf_to_shellcode.lib import five
import os
import stat

parser = ArgumentParser("ElfToShellcode")
parser.add_argument("--input", help="elf input file", required=True)
parser.add_argument("--arch",
                    required=True,
                    choices=Arches.__all__,
                    help="Elf file target architecture")
parser.add_argument("--endian",
                    required=True,
                    choices=ENDIANS,
                    help="Target elf file endian")
parser.add_argument("--output", default=None, help="Output file path")
parser.add_argument("--start-method", default=StartFiles.no_start_files,
                    choices=StartFiles.__all__, help="Start method required for full glibc usage")
parser.add_argument("--verbose", default=False, action="store_true", help="Verbose output")
parser.add_argument("--save-without-header", default=False, action="store_true",
                    help="Debug option, use only to store the elf without the mini loader and the relocation table")
parser.add_argument("--loader-supports",
                    choices=LoaderSupports.choices.keys(),
                    nargs="+",
                    required=False,
                    help="Loader additional features, this will increase the size of the static loader",
                    default=[])
parser.add_argument("--interactive",
                    default=False,
                    action="store_true",
                    help="Debug mode to open interactive cli with the shellcode class")
parser.add_argument("--output-format",
                    choices=OUTPUT_FORMATS,
                    required=False,
                    default='shelf',
                    help="Output format for shellcode, read more in the docs/output_format.md")
parser.add_argument("--loader-path",
                    help="Loader to use while creating the target shellcode",
                    default=None, required=False)
parser.add_argument("--loader-symbols-path",
                    required=False,
                    default=None,
                    help="Loader symbols to use while creating the shellcode"
                    )
args = parser.parse_args()

if any([args.loader_path, args.loader_symbols_path]) and not all([args.loader_path, args.loader_symbols_path]):
    parser.error("--loader-path and --loader-symbols-path must be used together")
    sys.exit(1)

sys.modules["global_args"] = args
if args.verbose:
    logging.basicConfig(level=logging.DEBUG)
    logging.info("Verbose level: DEBUG")
if args.output:
    output_file = args.output
else:
    output_file = args.input + "{0}.out.shell".format(args.arch)

with open(output_file, "wb") as fp:
    shellcode = make_shellcode(args.input, arch=args.arch, endian=args.endian,
                               start_file_method=args.start_method)
    fp.write(five.to_file(shellcode))
    st = os.stat(output_file)
    os.chmod(output_file, st.st_mode | stat.S_IEXEC)

print("Created: {}".format(output_file))



from elf_to_shellcode.lib.consts import RelocationAttributes, StartFiles
import logging


class IrelativeRelocs(object):
    def __init__(self, irelative_type,
                 jmp_slot_type=None,
                 get_glibc_instructions_filter=None):
        self.irelative_type = irelative_type
        self.jmp_slot_type = jmp_slot_type
        self.get_glibc_instructions_filter = get_glibc_instructions_filter
        self.glibc_irelative_first_reference = 2 ** 32
        self.glibc_last_reference = 0
        self.logger = logging.getLogger("[IRELATIVE-HELPER]")

    def relocation_for_rela_plt_got_plt(self, shellcode, shellcode_data):
        """
        Specific handler for the .rela.plt and .got.plt relocations
        :return:
        """
        rela_plt = shellcode.elffile.get_section_by_name('.rela.plt')
        got_plt = shellcode.elffile.get_section_by_name(".got.plt")
        if not rela_plt:
            return shellcode_data
        if rela_plt and not got_plt:
            raise Exception("Relocation not supported yet")

        for relocation in rela_plt.iter_relocations():
            if relocation.entry.r_info != self.irelative_type:
                self.logger.error("Relocation not supported yet: {}".format(
                    relocation.entry.r_info
                ))
                continue

            virtual_offset = shellcode.make_relative(relocation.entry.r_offset)
            function_offset = shellcode.make_relative(relocation.entry.r_addend)
            self.logger.info("| IRELATIVE | Relative(*{}={}()) Absolute(*{}={}())".format(
                hex(virtual_offset),
                hex(function_offset),
                hex(shellcode.make_absolute(virtual_offset)),
                hex(shellcode.make_absolute(function_offset))
            ))
            shellcode.addresses_to_patch[virtual_offset] = [function_offset,
                                                            RelocationAttributes.call_to_resolve]
        return shellcode_data

    def relocation_for_rel_plt_got_plt(self, shellcode, shellcode_data):
        """
        Specific handler for the .rela.plt and .got.plt relocations
        :return:
        """
        rel_plt = shellcode.elffile.get_section_by_name('.rel.plt')
        got_plt = shellcode.elffile.get_section_by_name(".got.plt")
        if not rel_plt:
            return shellcode_data
        if rel_plt and not got_plt:
            raise Exception("Relocation not supported yet")

        for relocation in rel_plt.iter_relocations():
            relocation_type = relocation.entry.r_info_type
            if relocation_type == self.irelative_type:

                self.do_irelative_rel_plt_got_plt(
                    shellcode=shellcode,
                    got_plt=got_plt,
                    relocation=relocation
                )
            elif relocation_type == self.jmp_slot_type:
                self.do_jmp_slot_relocation(
                    shellcode=shellcode,
                    got_plt=got_plt,
                    relocation=relocation,
                    shellcode_data=shellcode_data
                )
            else:
                raise Exception("Relocation not supported yet: {}".format(
                    relocation.entry.r_info_type
                ))
        self.fix_glibc_references(shellcode)
        return shellcode_data

    def fix_glibc_references(self, shellcode):
        if shellcode.start_file_method != StartFiles.glibc:
            return

        assert self.glibc_irelative_first_reference != 2 ** 32
        assert self.glibc_last_reference != 0

        """ here we should search for the opcode containing first reference, last reference
             and fix thoose opcodes
             ...eg :
             .text:0804A480 C7 C3 B4 81 04 08                       mov     ebx, offset off_80481B4
             .text:0804A486 C7 C7 24 82 04 08                       mov     edi, 8048224h
        """

        for entry in [self.glibc_irelative_first_reference, self.glibc_last_reference]:
            addresses = shellcode.disassembler.get_instruction_addresses(
                instruction_filter=self.get_glibc_instructions_filter(entry)
            )
            for address in addresses:
                self.logger.info("[!GLIBC] |InstructionPatch| Relative({}), Absolute({})".format(
                    hex(address),
                    hex(shellcode.make_absolute(address))
                ))
                shellcode.addresses_to_patch[address] = shellcode.make_relative(entry)

    def do_irelative_rel_plt_got_plt(self, shellcode, got_plt, relocation):
        virtual_offset = shellcode.make_relative(relocation.entry.r_offset)
        function_offset_rl = shellcode.address_utils.section_get_ptr_at_address(
            section=got_plt,
            address=relocation.entry.r_offset,
            alignment=got_plt.header.sh_entsize
        )
        relocation_entry_relative = shellcode.make_relative(relocation.entry.r_offset)
        function_offset = shellcode.make_relative(function_offset_rl)

        hdr = "GLIBC_R"

        if shellcode.start_file_method != StartFiles.glibc:
            hdr = "IRELATIVE"
            shellcode.addresses_to_patch[virtual_offset] = [function_offset,
                                                            RelocationAttributes.call_to_resolve]
        else:
            shellcode.addresses_to_patch[virtual_offset] = function_offset

        self.logger.info("| {} | Relative(*{}={}()) Absolute(*{}={}())".format(
            hdr,
            hex(virtual_offset),
            hex(function_offset),
            hex(shellcode.make_absolute(virtual_offset)),
            hex(shellcode.make_absolute(function_offset))
        ))
        if shellcode.start_file_method == StartFiles.glibc:
            """
                If so the elf header contain references to those function, 
                we must found those references,
                
            """
            stop_at = shellcode.instruction_offset_after_objdump
            for index in xrange(0, stop_at, shellcode.ptr_size):
                entry = shellcode.unpack_ptr(
                    shellcode.shellcode_data[index: index + shellcode.ptr_size])
                # Here we try to locate this reference.
                if entry == relocation.entry.r_offset:
                    self.logger.info("| HEADER | Relative(*{}={}()) Absolute(*{}={}())".format(
                        hex(index),
                        hex(relocation_entry_relative),
                        hex(shellcode.make_absolute(index)),
                        hex(shellcode.make_absolute(relocation_entry_relative))
                    ))
                    shellcode.addresses_to_patch[index] = relocation_entry_relative
                    address_not_relative = shellcode.linker_base_address + index

                    self.glibc_irelative_first_reference = min(self.glibc_irelative_first_reference,
                                                               address_not_relative)
                    self.glibc_last_reference = max(self.glibc_irelative_first_reference,
                                                    address_not_relative + shellcode.ptr_size * 2)

    def do_jmp_slot_relocation(self, shellcode,
                               got_plt,
                               relocation,
                               shellcode_data):
        # Already handled
        pass



from elf_to_shellcode.lib.consts import RelocationAttributes
from elf_to_shellcode.lib.consts import RELOC_TYPES
import logging


class DynamicRelocations(object):
    def __init__(self, reloc_types):
        self.handlers = {
            'JMPREL': self.handle_jmp_slot_relocs,
            "REL": self.handle_rels
        }
        self.logger = logging.getLogger(self.__class__.__name__)
        self.reloc_types = reloc_types

        self.entry_handlers = {
            self.reloc_types[RELOC_TYPES.RELATIVE]: self.reloc_relative_handle,
            self.reloc_types[RELOC_TYPES.GLOBAL_SYM]: self.global_sym_dat_handle,
            self.reloc_types[RELOC_TYPES.GLOBAL_DAT]: self.global_sym_dat_handle,
            self.reloc_types[RELOC_TYPES.JMP_SLOT]: self.jmp_slot_reloc_handle

        }

    def call_entry_handler(self, entry, shellcode, dynsym):
        if entry.r_info_type in self.reloc_types[RELOC_TYPES.DO_NOT_HANDLE]:
            self.logger.warn("Not calling handler for entry: {}, marked as do not handle".format(
                entry
            ))
            return
        entry_handler = self.entry_handlers.get(entry.r_info_type, None)
        if not entry_handler:
            self.logger.error("Entry handler for: {} not found, available: {}".format(
                entry,
                self.entry_handlers.keys()
            ))
            assert False

        entry_handler(entry=entry, shellcode=shellcode, dynsym=dynsym)

    def handle(self, shellcode, shellcode_data):

        # This case is already integrated in to the default relocation algorithm
        dynsym = shellcode.elffile.get_section_by_name(".dynsym")
        dynamic = shellcode.elffile.get_section_by_name('.dynamic')
        if not dynamic:
            return shellcode_data
        assert dynsym
        relocation_table = dynamic.get_relocation_tables()
        relocs = relocation_table.keys()
        for reloc_type in relocs:
            handler = self.handlers.get(reloc_type, None)
            if not handler:
                self.logger.error("[HandlerNotFound] {}".format(
                    reloc_type
                ))
                raise Exception("Not supported")
            else:
                logging.info("Relocation types: {}".format(
                    relocation_table.keys()
                ))
                handler(shellcode=shellcode,
                        table=relocation_table[reloc_type],
                        dynsym=dynsym)
        self.handle_other_dynamic_relocations(shellcode=shellcode)
        return shellcode_data

    def handle_other_dynamic_relocations(self, shellcode):
        rel_dyn = shellcode.elffile.get_section_by_name('.rel.dyn')
        rel_plt = shellcode.elffile.get_section_by_name('.rel.plt')
        if rel_dyn:
            self.handle_rel_dyn(shellcode, rel_dyn)

    def reloc_relative_handle(self, shellcode, entry, **kwargs):
        offset = shellcode.make_relative(entry.r_offset)
        self.logger.info("[REL_RELATIVE] Relative({}) Absolute({})".format(
            hex(offset),
            hex(shellcode.make_absolute(offset)),
        ))
        shellcode.addresses_to_patch[offset] = [0, RelocationAttributes.relative]

    def global_sym_dat_handle(self, shellcode, dynsym, entry):
        sym = dynsym.get_symbol(entry.r_info_sym)
        offset = shellcode.make_relative(entry.r_offset)
        r_address = shellcode.make_relative(sym.entry.st_value)
        self.logger.info("[SYM_R|{}] Relative(*{}={}) Absolute(*{}={})".format(
            sym.name,
            hex(offset),
            hex(r_address),
            hex(shellcode.make_absolute(offset)),
            hex(shellcode.make_absolute(r_address))
        ))
        shellcode.addresses_to_patch[offset] = r_address

    def handle_rel_dyn(self, shellcode, rel_dyn):
        dynsym = shellcode.elffile.get_section_by_name(".dynsym")

        for entry in rel_dyn.iter_relocations():
            entry = entry.entry
            self.call_entry_handler(
                entry=entry,
                dynsym=dynsym,
                shellcode=shellcode
            )

    def jmp_slot_reloc_handle(self, shellcode, entry, dynsym):
        entry = entry
        symbol_obj = dynsym.get_symbol(entry.r_info_sym)
        v_offset = shellcode.make_relative(symbol_obj.entry.st_value)
        offset = shellcode.make_relative(entry.r_offset)
        sym_type = symbol_obj.entry.st_info.type

        if shellcode.loader_symbols.has_symbol(symbol_obj.name):
            jmp_slot_address = shellcode.loader_symbols.get_relative_symbol_address(
                symbol_name=symbol_obj.name
            )
            self.logger.info("[JMP_SL] Relative(*{}={}()) Absolute(*{}={}())".format(
                hex(offset),
                hex(jmp_slot_address),
                hex(shellcode.make_absolute(offset)),
                hex(shellcode.make_absolute(jmp_slot_address))
            ))
            shellcode.addresses_to_patch[offset] = [jmp_slot_address,
                                                    RelocationAttributes.relative_to_loader_base]
            return

        if not symbol_obj.entry.st_value:
            name = symbol_obj.name
            if name not in [
                '_pthread_cleanup_pop_restore',
                '__pthread_unwind',
                '_Unwind_Resume'
            ]:
                self.logger.error("Symbol not found: {}".format(name))
                assert False
            else:
                self.logger.error("LIBC Symbol not found: {}".format(name))
        if sym_type == "STT_FUNC":
            shellcode.addresses_to_patch[offset] = v_offset
        elif sym_type == "STT_LOOS":
            shellcode.addresses_to_patch[offset] = [v_offset,
                                                    RelocationAttributes.call_to_resolve]
        elif sym_type == "STT_NOTYPE":
            self.logger.error("Sym type: STT_NOTYPE error")
        else:
            self.logger.error("Unknown symtype: {} {}".format(symbol_obj.name, sym_type))
            assert False
        self.logger.info("[JMP_SL_{}] Relative(*{}={}()) Absolute(*{}={}())".format(
            sym_type,
            hex(offset),
            hex(v_offset),
            hex(shellcode.make_absolute(offset)),
            hex(shellcode.make_absolute(v_offset))
        ))

    def handle_jmp_slot_relocs(self, shellcode,
                               table,
                               dynsym):
        for relocation in table.iter_relocations():
            entry = relocation.entry
            self.jmp_slot_reloc_handle(
                shellcode=shellcode,
                entry=entry,
                dynsym=dynsym
            )

    def handle_rels(self,
                    shellcode,
                    table,
                    dynsym):
        for entry in table.iter_relocations():
            entry = entry.entry
            self.call_entry_handler(
                entry=entry,
                dynsym=dynsym,
                shellcode=shellcode
            )



import os

base_file = os.path.dirname(__file__)


def get_resource(resource_name, resolve=True):
    if resolve:
        path = get_resource_path(resource_name)
    else:
        path = resource_name
    with open(path, "rb") as fp:
        return fp.read()


def get_resource_path(resource_name):
    return os.path.join(base_file, resource_name)



from elf_to_shellcode.lib.shellcode import Shellcode, create_make_shellcode
from elftools.elf.enums import ENUM_RELOC_TYPE_AARCH64
from elf_to_shellcode.lib.ext.irelative_relocations import IrelativeRelocs


# Refernce: https://static1.squarespace.com/static/59c4375b8a02c798d1cce06f/t/59d55a7bf5e2319471bb94a4/1507154557709/ELF+for+ARM64.pdf

class ArmX64Shellcode(Shellcode):
    def __init__(self, elffile, shellcode_data, endian, **kwargs):
        super(ArmX64Shellcode, self).__init__(
            elffile=elffile,
            shellcode_data=shellcode_data,
            endian=endian,
            arch="arm64",
            mini_loader_little_endian="mini_loader_arm_x64{}.shellcode",
            mini_loader_big_endian=None,
            shellcode_table_magic=0x8899aabbccddeeff,
            ptr_fmt="Q",
            sections_to_relocate={
                '.data.rel.ro': {'align_by': 'sh_addralign'},
            },
            support_dynamic=False,
            **kwargs
        )
        self.irelative = IrelativeRelocs(ENUM_RELOC_TYPE_AARCH64['R_AARCH64_TLS_DTPMOD32'])
        self.add_relocation_handler(self.irelative.relocation_for_rela_plt_got_plt)

    def build_shellcode_from_header_and_code(self, header, code):
        # Now we are going to align our shellcode
        aarch64_alignment = (2 << 12)
        if len(header) > aarch64_alignment:
            alignment = len(header) % aarch64_alignment
        else:
            alignment = aarch64_alignment - len(header)
        padding = b'\x00' * alignment
        header_moved = self.move_header_by_offset(header,
                                                  offset=len(padding))

        constructed = header_moved + padding + code
        return constructed


arm_x64_make_shellcode = create_make_shellcode(ArmX64Shellcode)



from elf_to_shellcode.lib.shellcode import Shellcode, create_make_shellcode
from elftools.elf.enums import ENUM_RELOC_TYPE_x64
from elf_to_shellcode.lib.ext.irelative_relocations import IrelativeRelocs


class IntelX64Shellcode(Shellcode):
    def __init__(self, elffile, shellcode_data, endian, **kwargs):
        super(IntelX64Shellcode, self).__init__(
            elffile=elffile,
            shellcode_data=shellcode_data,
            endian=endian,
            arch="x64",
            mini_loader_little_endian="mini_loader_x64{}.shellcode",
            mini_loader_big_endian=None,
            shellcode_table_magic=0x8899aabbccddeeff,
            ptr_fmt="Q",

            sections_to_relocate={
                '.data.rel.ro': {'align_by': 'sh_addralign'},
            },
            support_dynamic=False,
            **kwargs
        )
        self.irelative = IrelativeRelocs(ENUM_RELOC_TYPE_x64['R_X86_64_IRELATIVE'])
        self.add_relocation_handler(self.irelative.relocation_for_rela_plt_got_plt)


intel_x64_make_shellcode = create_make_shellcode(IntelX64Shellcode)



from elf_to_shellcode.resources import get_resource_path


class ShellcodeLoader(object):
    def __init__(self, symbols_path, loader_size):
        self.loader_symbols = symbols_path
        self.symbols = {}
        self.base_address = 0
        with open(self.loader_symbols) as fp:
            for symbol in fp.readlines():
                try:
                    symbols = [sym for sym in symbol.split(" ") if sym]
                    name = symbols[-1]
                    address = int(symbols[1], 16)
                    self.symbols[name.strip()] = address
                except:
                    pass
        self.loader_size = loader_size
        self.base_address = self.symbols['loader_main']

    def get_relative_symbol_address(self, symbol_name):
        sym_rel = self.get_symbol_address(symbol_name) - self.base_address
        return sym_rel

    def has_symbol(self, symbol_name):
        return symbol_name in self.symbols

    def get_symbol_address(self, symbol_name):
        return self.symbols[symbol_name]


from setuptools import find_packages, setup
import os
import sys

version = int(sys.version[0])
if version == 2:
    raise Exception("Only supported in python3")
py_specific_req_2 = [
    'lief==0.9',
]
py_specific_req_3 = [
    'lief==0.12.1',
]

py_specific_req = py_specific_req_2

if version >= 3:
    py_specific_req = py_specific_req_3
try:
    with open(os.path.join(os.path.dirname(__file__), 'README.md'), 'rb') as fp:
        README = str(fp.read())
except:
    print("Readme error")

setup(
    name='elf_to_shellcode',
    version='2.1.6',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
                         'pyelftools',
                         'capstone',
                     ] + py_specific_req,
    license='MIT License',
    description='Python package to create shellcdoes from elfs supported arch '
                '(mips, arm (32bit), i386 32bit, i386 64bit, aarch64)',
    long_description=README,
    long_description_content_type='text/markdown',
    url='https://github.com/jonatanSh/elf_to_shellcode',
    author='Jonathan Shimon',
    author_email='jonatanshimon@gmail.com',
    package_data={'': ['*.shellcode']}

)



import os

from elftools.elf.elffile import ELFFile
from elf_to_shellcode.resources import get_resource, get_resource_path
import struct
import sys
from elf_to_shellcode.lib.utils.address_utils import AddressUtils
from elf_to_shellcode.lib.consts import StartFiles, OUTPUT_FORMAT_MAP
from elf_to_shellcode.lib.utils.disassembler import Disassembler
from elf_to_shellcode.lib.ext.loader_symbols import ShellcodeLoader
from elf_to_shellcode.lib.ext.dynamic_symbols import DynamicRelocations
import logging
from elftools.elf.constants import P_FLAGS
from elf_to_shellcode.lib import five
import lief
import tempfile

PTR_SIZES = {
    4: "I",
    8: "Q"
}


class Shellcode(object):
    def __init__(self, elffile,
                 shellcode_data,
                 endian,
                 arch,
                 start_file_method,
                 mini_loader_big_endian,
                 mini_loader_little_endian,
                 shellcode_table_magic,
                 ptr_fmt,
                 support_dynamic=False,
                 sections_to_relocate={},
                 ext_bindings=[],
                 supported_start_methods=[],
                 reloc_types={}):
        self.support_dynamic = support_dynamic
        self.logger = logging.getLogger("[{}]".format(
            self.__class__.__name__
        ))

        self.elffile = elffile
        self.shellcode_table_magic = shellcode_table_magic
        # Key is the file offset, value is the offset to correct to
        self.addresses_to_patch = {}
        assert endian in ["big", "little"]
        self._loader = None  # Default No loader is required
        self.sections_to_relocate = sections_to_relocate

        self.shellcode_data = shellcode_data
        for segment in self.elffile.iter_segments():
            if segment.header.p_type in ['PT_LOAD']:
                self.linker_base_address = segment.header.p_vaddr
                break
        self.ptr_fmt = ptr_fmt
        self.ptr_signed_fmt = self.ptr_fmt.lower()
        self.relocation_handlers = []

        for binding in ext_bindings:
            get_binding, arguments = binding[0], binding[1]
            self.add_relocation_handler(get_binding(*arguments))
        self.supported_start_methods = supported_start_methods
        if StartFiles.no_start_files not in self.supported_start_methods:
            self.supported_start_methods.append(
                StartFiles.no_start_files
            )
        self.start_file_method = start_file_method
        assert self.start_file_method in self.supported_start_methods, "Error, start method: {} not supported for arch, supported methods: {}".format(
            self.start_file_method,
            self.supported_start_methods
        )
        self.address_utils = AddressUtils(unpack_size=self.unpack_size)

        if endian == "big":
            self.endian = ">"
            if mini_loader_big_endian:
                self.loader_path = self.format_loader(mini_loader_big_endian)
        else:
            if mini_loader_little_endian:
                self.loader_path = self.format_loader(mini_loader_little_endian)

            self.endian = "<"

        if self.args.loader_path:
            self.logger.info("Using loader resources from user")
            self.loader_path = self.args.loader_path
            self.loader_symbols_path = self.args.loader_symbols_path
        else:
            self.loader_path = get_resource_path(self.loader_path)
            self.loader_symbols_path = self.loader_path + ".symbols"

        assert self.loader_path
        assert self.loader_symbols_path
        self._loader = get_resource(self.loader_path, resolve=False)
        self.loader_symbols = ShellcodeLoader(self.loader_symbols_path,
                                              loader_size=len(self._loader))
        self.arch = arch
        self.debugger_symbols = [
            "loader_main"
        ]

        self.disassembler = Disassembler(self)
        if self.support_dynamic:
            self.dynamic_relocs = DynamicRelocations(reloc_types)
            self.add_relocation_handler(self.dynamic_relocs.handle)

    def format_loader(self, ld):
        if StartFiles.no_start_files == self.start_file_method:
            ld_base = ""
        elif StartFiles.glibc == self.start_file_method:
            ld_base = "_glibc"
        else:
            raise Exception("Unknown start method: {}".format(
                self.start_file_method
            ))
        args = sys.modules["global_args"]
        features_map = sorted(args.loader_supports, key=lambda lfeature: lfeature[1])
        for feature in features_map:
            value = getattr(self, "support_{}".format(feature))
            if not value:
                raise Exception("Arch does not support: {}".format(feature))
        loader_additional = "_".join([feature for feature in features_map])
        if loader_additional:
            loader_additional = "_" + loader_additional
        if self.args.output_format == OUTPUT_FORMAT_MAP.eshelf:
            loader_additional += "_eshelf"
        ld_name = ld.format(ld_base + loader_additional)

        self.logger.info("Using loader: {}".format(ld_name))
        return ld_name

    def make_absolute(self, address):
        return address + self.linker_base_address

    def add_relocation_handler(self, func):
        self.relocation_handlers.append(func)

    def pack(self, fmt, n):
        return struct.pack("{}{}".format(self.endian, fmt), n)

    def pack_pointer(self, n):
        return self.pack(self.ptr_fmt, n)

    def pack_list_of(self, lst, fmt):
        packed = five.py_obj()
        for item in lst:
            packed += self.pack(fmt, item)
        return packed

    def pack_list_of_pointers(self, lst):
        packed = five.py_obj()
        for item in lst:
            packed += self.pack_pointer(item)
        return packed

    def unpack_size(self, data, size):
        ptr_size = PTR_SIZES[size]
        return struct.unpack("{}{}".format(
            self.endian,
            ptr_size
        ), data)[0]

    @property
    def ptr_size(self):
        if self.ptr_fmt == "I":
            return 4
        if self.ptr_fmt == "Q":
            return 8
        raise Exception("Unknown ptr size")

    def sizeof(self, tp):
        if tp == "short":
            return 2
        else:
            raise NotImplementedError()

    @property
    def loader(self):
        if not self._loader:
            raise Exception("No loader for arch+endianes")
        assert self.pack_pointer(self.shellcode_table_magic) not in self._loader
        return self._loader

    @property
    def relocation_table(self):
        table = five.py_obj()

        for key, value in self.addresses_to_patch.items():
            if type(value) is not list:
                value = [value]

            value_packed = self.pack_list_of_pointers(value)

            relocation_entry = self.pack_pointer(key)
            relocation_entry += value_packed

            relocation_size = self.pack_pointer(len(relocation_entry) + self.ptr_size)
            relocation_entry = relocation_size + relocation_entry
            table += relocation_entry

        size_encoded = self.pack_pointer(len(table))
        return self.pack_pointer(self.shellcode_table_magic) + size_encoded + self.pre_table_header + table

    @property
    def pre_table_header(self):
        header = five.py_obj()
        sht_entry_header_size = 2 * self.sizeof("short")  # two shorts
        header += self.pack_pointer(
            self.elffile.header.e_ehsize + sht_entry_header_size
        )
        header += self.pack_pointer(len(self.loader))
        return header

    def correct_symbols(self, shellcode_data):
        for section, attributes in self.sections_to_relocate.items():
            self.section_build_relocations_table(
                section_name=section,
                align_attr=attributes['align_by'],
                relocate_all=attributes.get("relocate_all", False),
                shellcode_data=shellcode_data
            )
        return shellcode_data

    def section_build_relocations_table(self, section_name, align_attr, relocate_all, shellcode_data):
        data_section = self.elffile.get_section_by_name(section_name)
        original_symbol_addresses = self.get_original_symbols_addresses()
        if data_section:
            data_section_header = data_section.header

            for data_section_start in range(data_section_header.sh_offset,
                                            data_section_header.sh_offset + data_section_header.sh_size,
                                            self.ptr_size):
                data_section_end = data_section_start + self.ptr_size
                data_section_value = \
                    struct.unpack("{}{}".format(self.endian, self.ptr_fmt),
                                  shellcode_data[data_section_start:data_section_end])[0]
                if data_section_value not in original_symbol_addresses and not relocate_all:
                    self.logger.info("[R_SKIPPED]{}".format(hex(data_section_value)))
                    continue
                sym_offset = data_section_value - self.linker_base_address
                if sym_offset < 0:
                    continue
                symbol_relative_offset = data_section_start - data_section_header.sh_offset
                virtual_offset = data_section_header.sh_addr - self.linker_base_address
                virtual_offset += symbol_relative_offset
                self.logger.info("|{}| Relative(*{}={}), Absolute(*{}={})".format(
                    section_name,
                    hex(virtual_offset),
                    hex(sym_offset),
                    hex(self.make_absolute(virtual_offset)),
                    hex(self.make_absolute(sym_offset)),
                ))
                self.addresses_to_patch[virtual_offset] = sym_offset

        return shellcode_data

    def do_objdump(self, data):
        new_binary = five.py_obj()
        for segment in self.elffile.iter_segments():
            if segment.header.p_type in ['PT_LOAD']:
                header = segment.header
                segment_size = header.p_memsz
                start = (header.p_vaddr - self.linker_base_address)
                end = start + segment_size
                f_start = header.p_offset
                f_end = f_start + header.p_filesz
                assert f_end <= len(data), "Error segment offset outside of data: {} {}".format(
                    hex(f_end),
                    hex(len(data))
                )
                # first we make sure this part is already filled
                new_binary = five.ljust(new_binary, end, b'\x00')
                segment_data = data[f_start:f_end]

                # Now we rewrite the segment data
                # We look at new binary as memory dump so we write using virtual addresses offsets
                new_binary = new_binary[:start] + segment_data + new_binary[start + len(segment_data):]
        return new_binary  # TODO check if the elf header is really required

    @property
    def instruction_offset_after_objdump(self):
        # This function return the offset for the first executable section
        min_s = 2 ** 32
        for segment in self.elffile.iter_segments():
            if segment.header.p_type in ['PT_LOAD']:
                header = segment.header
                if header.p_flags & P_FLAGS.PF_X:
                    min_s = min(min_s, header.p_offset)
        assert min_s != 2 ** 32
        return min_s

    def get_original_symbols_addresses(self):
        symtab = self.elffile.get_section_by_name(".symtab")
        addresses = []
        for sym in symtab.iter_symbols():
            address = sym.entry.st_value
            if address >= self.linker_base_address:
                addresses.append(address)

        return addresses

    def get_symbol_name_from_address(self, address):

        symtab = self.elffile.get_section_by_name(".symtab")
        for sym in symtab.iter_symbols():
            sym_address = sym.entry.st_value
            if sym_address == address:
                return sym.name

        return None

    def get_shellcode_header(self):
        original_entry_point = self.elffile.header.e_entry
        new_entry_point = (original_entry_point - self.linker_base_address)
        return struct.pack("{}{}".format(self.endian, self.ptr_fmt), new_entry_point)

    def build_shellcode_from_header_and_code(self, header, code):
        return header + code

    @property
    def args(self):
        return sys.modules["global_args"]

    def get_shellcode(self):
        shellcode_data = self.shellcode_data
        shellcode_header = self.get_shellcode_header()
        shellcode_data = self.correct_symbols(shellcode_data)
        for handler in self.relocation_handlers:
            shellcode_data = handler(shellcode=self,
                                     shellcode_data=shellcode_data)
        shellcode_data = self.do_objdump(shellcode_data)
        # This must be here !
        relocation_table = self.relocation_table
        if self.args.output_format == OUTPUT_FORMAT_MAP.eshelf:
            return self.build_eshelf(

                relocation_table=relocation_table,
                shellcode_header=shellcode_header,
                shellcode_data=shellcode_data
            )

        full_header = self.loader + relocation_table + shellcode_header
        if self.args.save_without_header:
            self.logger.info("Saving without shellcode table")
            return shellcode_data
        else:
            return self.build_shellcode_from_header_and_code(full_header, shellcode_data)

    def build_eshelf(self, relocation_table, shellcode_header, shellcode_data):
        shellcode_data = relocation_table + shellcode_header + shellcode_data
        loader = lief.parse(self.loader_path)
        segment = lief.ELF.Segment()
        segment.type = lief.ELF.SEGMENT_TYPES.LOAD
        rwx = lief.ELF.SEGMENT_FLAGS(lief.ELF.SEGMENT_FLAGS.R | lief.ELF.SEGMENT_FLAGS.W | lief.ELF.SEGMENT_FLAGS.X)
        segment.flags = rwx
        segment.content = bytearray(shellcode_data)
        segment = loader.add(segment)
        tmp_path = tempfile.mktemp(".out")
        elf_buffer = None
        try:
            loader.write(tmp_path)
            with open(tmp_path, "rb") as fp:
                elf_buffer = fp.read()
        except Exception as e:
            self.logger.error("Error: {}".format(e))
        finally:
            if os.path.exists(tmp_path):
                os.remove(tmp_path)
        assert elf_buffer is not None, "Error"
        loader_symbol_address = self.loader.find(self.pack_pointer(0xdeadbeff))
        assert loader_symbol_address == self.loader.rfind(self.pack_pointer(0xdeadbeff)), "Error found more then one " \
                                                                                          "occurrence"
        self.logger.info("Setting shellcode base address at: {}".format(
            hex(segment.virtual_address)
        ))
        shellcode_start = self.pack_pointer(segment.virtual_address)
        elf_buffer_with_address = elf_buffer[:loader_symbol_address]
        elf_buffer_with_address += shellcode_start
        elf_buffer_with_address += elf_buffer[loader_symbol_address + self.ptr_size:]
        return elf_buffer_with_address

    def make_relative(self, address):
        return address - self.linker_base_address

    def unpack_ptr(self, stream):
        return struct.unpack("{}{}".format(self.endian,
                                           self.ptr_fmt), stream[:self.ptr_size])[0]

    def stream_unpack_pointers(self, stream, num_of_ptrs):
        return struct.unpack("{}{}".format(self.endian,
                                           self.ptr_fmt * num_of_ptrs), stream[:self.ptr_size * num_of_ptrs])

    def get_loader_base_address(self, shellcode):
        loader_size = len(self.loader)
        table_length = len(self.relocation_table)
        offset = loader_size + table_length
        return self.unpack_ptr(shellcode[offset:offset + self.ptr_size])

    def set_loader_base_address(self, shellcode, new_base_address):
        loader_size = len(self.loader)
        table_length = len(self.relocation_table)
        offset = loader_size + table_length
        shellcode = shellcode[:offset] + self.pack_pointer(new_base_address) + shellcode[offset + self.ptr_size:]
        return shellcode

    def move_header_by_offset(self, header, offset):
        """
        This function move the shellcode header by offset.
        It actually parse the shellcode just like the loader and correct the offsets
        :param shellcode_data:
        :param offset:
        :return:
        """
        current_offset = 0
        loader_size = len(self.loader)
        # Skipping the loader
        current_offset += loader_size
        magic = self.unpack_ptr(header[current_offset:current_offset + self.ptr_size])
        assert magic == self.shellcode_table_magic, 'Error reading magic'
        current_offset += self.ptr_size  # skipping magic
        table_size = self.unpack_ptr(header[current_offset:current_offset + self.ptr_size])
        current_offset += self.ptr_size  # skip ptr size
        current_offset += len(self.pre_table_header)

        handled_size = 0
        while handled_size < table_size:
            size, voff1, voff2 = self.stream_unpack_pointers(
                header[current_offset:],
                3
            )
            voff1 += offset
            voff2 += offset
            voff1 = self.pack_pointer(voff1)
            voff2 = self.pack_pointer(voff2)
            header_next = header[current_offset + self.ptr_size * 3:]
            header = header[:current_offset + self.ptr_size] + voff1 + voff2 + header_next

            current_offset += size
            handled_size += size
        entry_point = self.unpack_ptr(header[current_offset:current_offset + self.ptr_size])
        entry_point += offset
        entry_point = self.pack_pointer(entry_point)
        header = header[:current_offset] + entry_point + header[current_offset + self.ptr_size:]
        return header


def get_shellcode_class(elf_path, shellcode_cls, endian,
                        start_file_method):
    fd = open(elf_path, 'rb')
    elffile = ELFFile(fd)
    with open(elf_path, "rb") as fp:
        shellcode_data = fp.read()
    shellcode = shellcode_cls(elffile=elffile,
                              shellcode_data=shellcode_data,
                              endian=endian,
                              start_file_method=start_file_method)
    return shellcode, fd


def make_shellcode(elf_path, shellcode_cls, endian,
                   start_file_method):
    shellcode, fd = get_shellcode_class(elf_path, shellcode_cls, endian,
                                        start_file_method=start_file_method)
    args = sys.modules["global_args"]
    if args.interactive:
        print("Opening interactive shell")
        import IPython
        IPython.embed()
        sys.exit(1)
    shellcode = shellcode.get_shellcode()

    fd.close()
    return shellcode


def create_make_shellcode(shellcode_cls):
    def wrapper(elf_path, endian, start_file_method):
        return make_shellcode(elf_path, shellcode_cls, endian,
                              start_file_method=start_file_method)

    return wrapper


def not_supported_yet():
    def wrapper(elf_path, endian):
        raise Exception("Arch not supported yet")

    return wrapper



from elf_to_shellcode.lib.shellcode import Shellcode, create_make_shellcode
from elftools.elf.enums import ENUM_RELOC_TYPE_i386
from elf_to_shellcode.lib.ext.irelative_relocations import IrelativeRelocs
from elf_to_shellcode.lib.consts import StartFiles
from elf_to_shellcode.lib.consts import RELOC_TYPES


def get_glibc_instructions_filter(address):
    def _filter(instruction):
        if hex(address) in instruction.op_str:
            return True
        return False

    return _filter


class IntelX32Shellcode(Shellcode):
    def __init__(self, elffile, shellcode_data, endian, **kwargs):
        super(IntelX32Shellcode, self).__init__(
            elffile=elffile,
            shellcode_data=shellcode_data,
            endian=endian,
            arch="x32",
            mini_loader_little_endian="mini_loader_x32{}.shellcode",
            mini_loader_big_endian=None,
            shellcode_table_magic=0xaabbccdd,
            ptr_fmt="I",
            sections_to_relocate={
                '.data.rel.ro': {'align_by': 'sh_addralign'},

            },
            supported_start_methods=[
                StartFiles.no_start_files,
                # StartFiles.glibc
            ],
            reloc_types={
                RELOC_TYPES.JMP_SLOT: ENUM_RELOC_TYPE_i386['R_386_JUMP_SLOT'],
                RELOC_TYPES.RELATIVE: ENUM_RELOC_TYPE_i386['R_386_RELATIVE'],
                RELOC_TYPES.GLOBAL_SYM: ENUM_RELOC_TYPE_i386['R_386_32'],
                RELOC_TYPES.GLOBAL_DAT: ENUM_RELOC_TYPE_i386['R_386_GLOB_DAT'],
                RELOC_TYPES.DO_NOT_HANDLE: [
                    ENUM_RELOC_TYPE_i386['R_386_PC32'],
                    ENUM_RELOC_TYPE_i386['R_386_TLS_TPOFF'],
                ]

            },
            support_dynamic=True,
            **kwargs
        )
        self.irelative = IrelativeRelocs(
            irelative_type=ENUM_RELOC_TYPE_i386['R_386_IRELATIVE'],
            jmp_slot_type=ENUM_RELOC_TYPE_i386['R_386_JUMP_SLOT'],
            get_glibc_instructions_filter=get_glibc_instructions_filter
        )
        self.add_relocation_handler(self.irelative.relocation_for_rel_plt_got_plt)

        if self.start_file_method == StartFiles.glibc:
            self.add_relocation_handler(self.glibc_patch_symbols)

    def glibc_patch_symbols(self, shellcode, shellcode_data):
        """
        Glibc smybols:
         __environ
         __libc_stack_end
         for some reason doesn't support pic mode.
         therefor we manually relocate them
        :param shellcode: shellcode object
        :param shellcode_data: shellcode data
        :return: new shellcode object
        """
        symtab = shellcode.elffile.get_section_by_name('.symtab')

        special_symbols = [s.name for s in symtab.iter_symbols()
                           if s.entry.st_info.bind == 'STB_GLOBAL'
                           and s.entry.st_info.type == 'STT_OBJECT']

        for sym_name in special_symbols:
            sym = symtab.get_symbol_by_name(sym_name)

            if not sym or len(sym) == 0:
                continue
            if len(sym) > 1:
                raise Exception("Unknown error")
            sym = sym[0]
            entry_address = sym.entry.st_value

            addresses = shellcode.disassembler.get_instruction_addresses(
                instruction_filter=get_glibc_instructions_filter(entry_address)
            )

            for address in addresses:
                self.logger.info("![GLIBC] |InstructionPatch| Sym({}) Relative({}), Absolute({})".format(
                    sym.name,
                    hex(address),
                    hex(shellcode.make_absolute(address))
                ))
                shellcode.addresses_to_patch[address] = self.make_relative(entry_address)
        return shellcode_data


intel_x32_make_shellcode = create_make_shellcode(IntelX32Shellcode)
